# Copyright 1999-2024. WebPros International GmbH. All rights reserved.

# This Python module is designed to implement specific changes for mailing lists.
#
# It is meant to be used with the 'mailman withlist' command, expecting
# the target list to be passed as an argument named 'mlist'. The 'mailman' utility
# will automatically pass the list when you use the '-l [listname]' argument with it.
#
# The module should only be called through the mailman utility, as it relies
# on the context prepared by the utility. Attempting to call the module in any
# other way may result in "uninitialized context" problems.
#
# Further documentation on the 'withlist' sub-command can be found at the link provided:
# https://docs.mailman3.org/projects/mailman/en/latest/src/mailman/commands/docs/shell.html

import itertools


def set_password(mlist, next_password):
    from mailman.database.transaction import transaction
    from mailman.config import config
    with transaction():
        mlist.moderator_password = config.password_context.encrypt(next_password).encode('us-ascii')


def set_encrypted_password(mlist, enc_next_password):
    from mailman.database.transaction import transaction
    with transaction():
        mlist.moderator_password = enc_next_password.encode('us-ascii')


def get_encrypted_password(mlist):
    print(mlist.moderator_password.decode('us-ascii'))


def get_owners(mlist):
    for owner_address in mlist.owners.addresses:
        print(owner_address)


def _substitute_old_email_domain(email, old_domain, next_domain):
    if not email.endswith("@" + old_domain):
        return email
    return email.replace("@" + old_domain, "@" + next_domain)


def _get_address(usermanager, address):
    addr_object = usermanager.get_address(address)
    if addr_object is None:
        addr_object = usermanager.create_address(address)
    return addr_object


def move_list_to_domain(mlist, listname, old_domain, next_domain):
    from mailman.database.transaction import transaction
    from mailman.interfaces.member import MemberRole
    from mailman.interfaces.usermanager import IUserManager
    from mailman.interfaces.domain import IDomainManager
    from zope.component import getUtility

    usermanager = getUtility(IUserManager)

    domain_manager = getUtility(IDomainManager)
    if domain_manager.get(next_domain) is None:
        domain_manager.add(next_domain)

    with transaction():
        # According to the developers of Mailman, modifying the list_id is essentially equivalent to creating a new list.
        # Therefore, in order to ensure that subscribers are not lost after changing the domain,
        # all items in the list must be resubscribed.
        # More information can be found in the official Mailman mailing list: https://lists.mailman3.org/archives/list/mailman-users@mailman3.org/thread/MPK76WPWUETCRBFJ3HUW23HWDDUWTO4U/
        subscribers = {
            MemberRole.owner: [addr.email for addr in mlist.owners.addresses],
            MemberRole.moderator: [addr.email for addr in mlist.moderators.addresses],
            MemberRole.member: [addr.email for addr in mlist.members.addresses]
        }

        for member in itertools.chain(mlist.owners.members, mlist.moderators.members, mlist.members.members):
            member.unsubscribe()

        mlist._list_id = listname + "." + next_domain
        mlist.mail_host = next_domain

        for role, addresses in subscribers.items():
            for address in addresses:
                subscriber = _get_address(usermanager, _substitute_old_email_domain(address, old_domain, next_domain))
                mlist.subscribe(subscriber, role)
